@@ -126,4 +126,53 @@ void dmEuclid(double *learn, double *valid, int *n, int *m, int *p, double *dm,
 
 
 
+void dmEuclid2(double *learn, double *valid, int *learnF, int *validF, int *n, int *m, int *p, int *p2, double *dm, int *cl, int *k, double *mink, double *weights, double *weights2){
+    int i, j, l, l2, t, nn, ii, kk; 
+    double tmp, *dvec, maxD, tmp2;
+    int *cvec;
+
+    kk = MAX(10L, *k);
+    kk = MIN(kk, *n);
+    nn = MIN(2L*kk, *n);
+
+    cvec = (int *) R_alloc(nn, sizeof(int));
+    dvec = (double *) R_alloc(nn, sizeof(double));
+    for(t=0;t<nn;t++)dvec[t]= big;
+    for(j=0;j<(*m);j++){
+        i=0L;
+        ii=0L; 
+        maxD = big;
+        while(i<*n){
+            tmp=0.0;
+            l=0; 
+            l2=0;
+	        while(l<*p && tmp < (maxD+eps)){
+                tmp2 = learn[i+l*n[0]]-valid[j+l*m[0]];
+	            tmp += (tmp2*tmp2)* weights[l];
+                l++;               
+	        }
+            while(l2<*p2 && tmp < (maxD+eps)){
+                if(learnF[i+l2*n[0]] != validF[j+l2*m[0]]) tmp += weights2[l2];
+                l2++;               
+	        }            
+            if(tmp < maxD){
+                dvec[ii]=tmp;
+                cvec[ii]=i;
+                ii++;
+            }
+            if( ii==(nn-1L) ){  
+                rsort_with_index(dvec, cvec, nn);
+                ii= *k-1L;
+                maxD = dvec[*k-1L]; 
+                }
+            i++;         
+	    }
+        rsort_with_index(dvec, cvec, nn);
+        for(t=0;t<*k;t++){
+            cl[j+t * *m]=cvec[t];
+            dm[j+t * *m]=sqrt(dvec[t]);
+        }
+    }
+}
+
 
